@@ -583,14 +583,19 @@ class Scenario(object):
         if tags is None:
             return True
 
-        if not self.tags:
+        has_exclusionary_tags = any([t.startswith('-') for t in tags])
+
+        if not self.tags and not has_exclusionary_tags:
             return False
 
         matched = []
 
-        for tag in self.tags:
-            if tag in tags:
-                return True
+        if isinstance(self.tags, list):
+            for tag in self.tags:
+                if tag in tags:
+                    return True
+        else:
+            self.tags = []
 
         for tag in tags:
             exclude = tag.startswith('-')
@@ -703,9 +708,11 @@ class Scenario(object):
 
         def try_finding_with(regex):
             found = regex.search(original_string)
+
             if found:
                 tag_lines = found.group().splitlines()
-                return list(chain(*map(self._extract_tag, tag_lines)))
+                tags = list(chain(*map(self._extract_tag, tag_lines)))
+                return tags
 
         for regex in regexes:
             found = try_finding_with(regex)
@@ -715,8 +722,9 @@ class Scenario(object):
         return []
 
     def _extract_tag(self, item):
-        regex = re.compile(r'[@](\S+)')
-        return regex.findall(item)
+        regex = re.compile(r'(?:(?:^|\s+)[@]([^@\s]+))')
+        found = regex.findall(item)
+        return found
 
     def _resolve_steps(self, steps, outlines, with_file, original_string):
         for outline in outlines:
